/*
This program is confidential and proprietary to MRJ Technology Solutions
and may not be reproduced, published or disclosed to others without
written authorization from MRJ.

Copyright (c) 1998 MRJ Technology Solutions, Inc. All Rights Reserved.

This program is derived from prior work some of which was originally
written as a joint project between the Numerical Aerospace Simulation
(NAS) Systems Division of NASA Ames Research Center and the National
Energy Research Supercomputer Center (NERSC) of Lawrence Livermore
National Laboratory.
*/
/*
 * qterm
 *  The qterm command terminates the batch server.
 *
 * Synopsis:
 *  qterm [-t type] [server ...]
 *
 * Options:
 *  -t  delay   Jobs are (1) checkpointed if possible; otherwise, (2) jobs are
 *		rerun (requeued) if possible; otherwise, (3) jobs are left to
 *              run.
 *
 *      immediate
 *              Jobs are (1) checkpointed if possible; otherwise, (2) jobs are
 *		rerun if possible; otherwise, (3) jobs are aborted.
 *
 *	quick
 *		The server will save state and exit leaving running jobs
 *		still running.  Good for shutting down when you wish to
 *		quickly restart the server.
 *
 * Arguments:
 *  server ...
 *      A list of servers to terminate.
 *
 * Written by:
 *  Bruce Kelly
 *  National Energy Research Supercomputer Center
 *  Livermore, CA
 *  May, 1993
 */

#include "cmds.h"
#include <pbs_config.h>   /* the master config generated by configure */

static char ident[] = "@(#) $RCSfile: qterm.c,v $ $Revision: 2.0 $";

int exitstatus = 0; /* Exit Status */

static void execute ();

main ( argc, argv )
int argc;
char **argv;
{
/*
 *  This routine sends a Server Shutdown request to the batch server.  If the
 * batch request is accepted, and the type is IMMEDIATE, then no more jobs
 * are accepted and all jobs are checkpointed or killed.  If the type is
 * DELAY, then only privileged users can submit jobs, and jobs will be
 * checkpointed if available.
 */

    static char opts[] = "t:";     /* See man getopt */
    int s;                  /* The execute line option */
    static char usage[] = "Usage: qterm [-t immediate|delay|quick] [server ...]\n";
    char *type = NULL;      /* Pointer to the type of termination */
    int manner;             /* The type of termination */
    int errflg = 0;         /* Error flag */
    
    /* Command line options */
    while ((s = getopt(argc, argv, opts)) != EOF)
        switch (s) {
        case 't':
            type = optarg;
            break;
        case '?':
        default:
            errflg++;
            break;
        }
    if ( errflg ) {
        fprintf(stderr, usage);
        exit(1);
    } else if ( type != NULL ) {
        if ( strcmp(type, "delay") == 0 )
            manner = SHUT_DELAY;
        else if ( strcmp(type, "immediate") == 0 )
            manner = SHUT_IMMEDIATE;
	else if ( strcmp(type, "quick") == 0)
	    manner = SHUT_QUICK;
        else {
            fprintf(stderr, usage);
            exit(1);
        }
    } else
        manner = SHUT_IMMEDIATE;

    if ( optind < argc )
        for ( ; optind < argc; optind++ )
            execute(manner, argv[optind]);
    else
	execute(manner, "");

    exit (exitstatus);
}


/*
 * void execute( int manner, char *server )
 *
 * manner   The manner in which to terminate the server.
 * server   The name of the server to terminate.
 *
 * Returns:
 *  None
 *
 * File Variables:
 *  exitstatus  Set to two if an error occurs.
 */
static void
execute( manner, server )
int manner;
char *server;
{
    int ct;         /* Connection to the server */
    int err;        /* Error return from pbs_terminate */
    char *errmsg;   /* Error message from pbs_terminate */
    
    if ( (ct = cnt2server(server)) > 0 ) {
        err = pbs_terminate(ct, manner, NULL);
        if ( err != 0 ) {
            errmsg = pbs_geterrmsg(ct);
            if ( errmsg != NULL ) {
                fprintf(stderr, "qterm: %s ", errmsg);
            } else {
                fprintf(stderr, "qterm: Error (%d) terminating server ", pbs_errno);
            }
            fprintf(stderr, "%s\n", server);
            exitstatus = 2;
        }
        pbs_disconnect(ct);
    } else {
        fprintf(stderr, "qterm: could not connect to server %s (%d)\n", server, pbs_errno);
        exitstatus = 2;
    }
}
